@@ -6,6 +6,7 @@ import vectorian.alignment
 import vectorian.embeddings
 import vectorian.metrics
 import vectorian.session
+import vectorian.core as core
 
 
 ROOT_LEVEL_STYLE = {'description_width': '10em'}
@@ -122,7 +123,7 @@ class PNormWidget:
 		return self._vbox
 
 	def make(self):
-		return vectorian.metrics.ModifiedMetric(
+		return vectorian.metrics.ModifiedVectorSimilarity(
 			vectorian.metrics.PNormDistance(p=self._p.value),
 			vectorian.metrics.RadialBasis(self._scale.value)
 		)
@@ -474,27 +475,27 @@ class SlidingGapCostWidget:
 
 
 class ConstantGapCostWidget(SlidingGapCostWidget):
-	def __init__(self, iquery):
-		super().__init__(iquery, 'Cost:', vectorian.alignment.ConstantGapCost)
+	def __init__(self, iquery, k=0.1):
+		super().__init__(iquery, 'Cost:', vectorian.alignment.ConstantGapCost, default=k)
 
 	def describe(self):
 		return "**constant gap cost** of **%.2f**" % self._cost.value
 
 
 class LinearGapCostWidget(SlidingGapCostWidget):
-	def __init__(self, iquery):
-		super().__init__(iquery, 'Cost:', vectorian.alignment.LinearGapCost)
+	def __init__(self, iquery, k=0.1):
+		super().__init__(iquery, 'Cost:', vectorian.alignment.LinearGapCost, default=k)
 
 	def describe(self):
 		return "**linear gap cost** of **%.2f**" % self._cost.value
 
 
-class ExponentialGapCostWidget(SlidingGapCostWidget):
-	def __init__(self, iquery):
-		super().__init__(iquery, 'Cutoff:', vectorian.alignment.ExponentialGapCost, default=3, max=20, step=1)
+class SmoothGapCostWidget(SlidingGapCostWidget):
+	def __init__(self, iquery, k=3):
+		super().__init__(iquery, 'Cutoff:', vectorian.alignment.smooth_gap_cost, default=k, max=k * 7, step=1)
 
 	def describe(self):
-		return "**exponential gap cost** with a 50%% cutoff at **%.2f**" % self._cost.value
+		return "**smooth gap cost** with a 50%% penalty at **%.2f**" % self._cost.value
 
 
 class GapCostWidget(FineTuneableWidget):
@@ -505,7 +506,7 @@ class GapCostWidget(FineTuneableWidget):
 	_types = [
 		('Constant', ConstantGapCostWidget),
 		('Linear', LinearGapCostWidget),
-		('Exponential', ExponentialGapCostWidget)
+		('Exponential', SmoothGapCostWidget)
 	]
 
 	_default = 'Linear'
@@ -537,10 +538,53 @@ class GapMaskWidget:
 		return f" (applied to {text})"
 
 
+def derive_gap_cost_args(gap_cost):
+	args_s = gap_cost['s'].to_tuple()
+	args_t = gap_cost['t'].to_tuple()
+	if args_s != args_t:
+		raise RuntimeError(
+			f'cannot derive non-unified gap costs for s and t: {args_s} != {args_t}')
+	if args_s[0] == 'exponential' and args_s[1] == 2:
+		return {
+			'default': 'Exponential',
+			'default_options': {
+				'k': int(1 / args_s[2])
+			}
+		}
+	elif args_s[0] == 'linear':
+		return {
+			'default': 'Linear',
+			'default_options': {
+				'k': args_s[1]
+			}
+		}
+	elif args_s[0] == 'constant':
+		return {
+			'default': 'Constant',
+			'default_options': {
+				'k': args_s[1]
+			}
+		}
+	else:
+		raise RuntimeError(f'cannot decompose gap cost specification {args_s}')
+
+
 class AlignmentAlgorithmWidget:
-	def __init__(self, iquery, parameters, indent='5em', similarity=None):
+	def __init__(self, iquery, klass, alignment=None, indent='5em', similarity=None):
+		if alignment is not None:
+			gap_cost_widget_args = derive_gap_cost_args(alignment.gap)
+		else:
+			gap_cost_widget_args = {
+				'default': 'Exponential'
+			}
+
+		self._gap_cost = GapCostWidget(iquery, **gap_cost_widget_args)
+		self._gap_mask = GapMaskWidget(iquery)
+
+		self._klass = klass
 		self._token_sim = TokenSimilarityMetricWidget(iquery, similarity)
 
+		parameters = [self._gap_cost.widget, self._gap_mask.widget]
 		if parameters is None:
 			self._vbox = widgets.VBox([
 				self._token_sim.widget])
@@ -562,76 +606,32 @@ class AlignmentAlgorithmWidget:
 	def describe_token_sim(self):
 		return self._token_sim.describe()
 
-	def describe_alignment(self):
-		raise NotImplementedError()
-
-
-class NeedlemanWunschWidget(AlignmentAlgorithmWidget):
-	def __init__(self, iquery, alignment=None, **kwargs):
-		self._gap_cost = GapCostWidget(iquery, fix_to="Linear")
-		self._gap_mask = GapMaskWidget(iquery)
-		super().__init__(
-			iquery, [self._gap_cost.widget, self._gap_mask.widget], **kwargs)
-
 	def make(self):
-		return vectorian.alignment.NeedlemanWunsch(
-			gap=self._gap_cost.make().to_scalar(),
-			gap_mask=self._gap_mask.get())
+		gap = self._gap_cost.make()
+		mask = self._gap_mask.get()
+		return self._klass(
+			gap={
+				's': gap if 's' in mask else vectorian.alignment.ConstantGapCost(0),
+				't': gap if 't' in mask else vectorian.alignment.ConstantGapCost(0)
+			})
 
 	def describe_alignment(self):
 		return "with " + self._gap_cost.describe() + self._gap_mask.describe()
 
 
-class SmithWatermanWidget(AlignmentAlgorithmWidget):
-	def __init__(self, iquery, alignment=None, **kwargs):
-		self._gap_cost = GapCostWidget(iquery, fix_to="Linear")
-		self._gap_mask = GapMaskWidget(iquery)
-		self._zero = widgets.BoundedFloatText(
-			value=0.25,
-			min=0,
-			max=1,
-			step=0.1,
-			description='Zero:',
-			disabled=False)
-		super().__init__(
-			iquery,
-			[self._gap_cost.widget, self._gap_mask.widget, self._zero],
-			**kwargs)
-
-	def make(self):
-		return vectorian.alignment.SmithWaterman(
-			gap=self._gap_cost.make().to_scalar(),
-			gap_mask=self._gap_mask.get(),
-			zero=self._zero.value)
-
-	def describe_alignment(self):
-		return "with " + self._gap_cost.describe() + self._gap_mask.describe() + (". Zero similarity is set to **%.2f**" % self._zero.value)
+class GlobalAlignmentWidget(AlignmentAlgorithmWidget):
+	def __init__(self, *args, **kwargs):
+		super().__init__(*args, klass=vectorian.alignment.GlobalAlignment, **kwargs)
 
 
-class WatermanSmithBeyerWidget(AlignmentAlgorithmWidget):
-	def __init__(self, iquery, alignment=None, **kwargs):
-		self._gap_cost = GapCostWidget(iquery, default="Exponential")
-		self._gap_mask = GapMaskWidget(iquery)
-		self._zero = widgets.BoundedFloatText(
-			value=0.25,
-			min=0,
-			max=1,
-			step=0.1,
-			description='Zero:',
-			disabled=False)
-		super().__init__(
-			iquery,
-			[self._gap_cost.widget, self._gap_mask.widget, self._zero],
-			**kwargs)
+class SemiGlobalAlignmentWidget(AlignmentAlgorithmWidget):
+	def __init__(self, *args, **kwargs):
+		super().__init__(*args, **kwargs, klass=vectorian.alignment.SemiGlobalAlignment)
 
-	def make(self):
-		return vectorian.alignment.WatermanSmithBeyer(
-			gap=self._gap_cost.make(),
-			gap_mask=self._gap_mask.get(),
-			zero=self._zero.value)
 
-	def describe_alignment(self):
-		return "with " + self._gap_cost.describe() + self._gap_mask.describe() + (". Zero similarity is set to **%.2f**" % self._zero.value)
+class LocalAlignmentWidget(AlignmentAlgorithmWidget):
+	def __init__(self, *args, **kwargs):
+		super().__init__(*args, **kwargs, klass=vectorian.alignment.LocalAlignment)
 
 
 class WordMoversDistanceWidget(AlignmentAlgorithmWidget):
@@ -716,30 +716,33 @@ class AlignmentWidget(FineTuneableWidget):
 	_description = 'Alignment:'
 
 	_types = [
-		('Needleman-Wunsch', NeedlemanWunschWidget),
-		('Smith-Waterman', SmithWatermanWidget),
-		('Waterman-Smith-Beyer', WatermanSmithBeyerWidget),
+		('Global Alignment', GlobalAlignmentWidget),
+		('Semiglobal Alignment', SemiGlobalAlignmentWidget),
+		('Local Alignment', LocalAlignmentWidget),
 		('Word Movers Distance (WMD)', WordMoversDistanceWidget),
 		('Word Rotators Distance (WRD)', WordRotatorsDistanceWidget)
 	]
 
-	_default = 'Waterman-Smith-Beyer'
+	_default = 'Local Alignment'
 
 	def __init__(self, iquery, alignment=None, similarity=None, **kwargs):
 		if 'default_options' not in kwargs:
 			kwargs['default_options'] = {}
 
 		if alignment is not None:
+			def map_alignment(args):
+				locality = args['options']['locality']
+				to_str = dict((v, k) for k, v in core.pyalign.Locality.__members__.items())
+				return f'{to_str[locality].capitalize()} Alignment'
+
 			mapping = {
-				'needleman-wunsch': 'Needleman-Wunsch',
-				'smith-waterman': 'Smith-Waterman',
-				'waterman-smith-beyer': 'Waterman-Smith-Beyer',
-				'word-movers-distance': 'Word Movers Distance (WMD)',
-				'word-rotators-distance': 'Word Rotators Distance (WRD)'
+				'pyalign': map_alignment,
+				'word-movers-distance': lambda args: 'Word Movers Distance (WMD)',
+				'word-rotators-distance': lambda args: 'Word Rotators Distance (WRD)'
 			}
 
 			args = alignment.to_args(iquery.partition)
-			kwargs['default'] = mapping[args['algorithm']]
+			kwargs['default'] = mapping[args['algorithm']](args)
 			kwargs['default_options']['alignment'] = alignment
 
 		if similarity is not None:
